@@ -1,4 +1,4 @@
-# Copyright (C) 1998 by the Free Software Foundation, Inc.
+# Copyright (C) 1998,1999,2000 by the Free Software Foundation, Inc.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -24,12 +24,11 @@ is necessary to distinguish from default assignments done in the .InitVars()
 methods, before .CheckVersion() is called.
 
 For new versions you should add sections to the UpdateOldVars() and the
-UpdateOldUsers() sections, to preserve the sense of settings across
-structural changes.  Note that the routines have only one pass - when
-.CheckVersions() finds a version change it runs this routine and then
-updates the data_version number of the list, and then does a .Save(), so
-the transformations won't be run again until another version change is
-detected.
+UpdateOldUsers() sections, to preserve the sense of settings across structural
+changes.  Note that the routines have only one pass - when .CheckVersions()
+finds a version change it runs this routine and then updates the data_version
+number of the list, and then does a .Save(), so the transformations won't be
+run again until another version change is detected.
 
 """
 
@@ -37,17 +36,31 @@ detected.
 import re
 import string
 from types import ListType, StringType
-import mm_cfg
-import Utils
+
+from Mailman import mm_cfg
+from Mailman import Utils
+from Mailman import Message
+from Mailman.Logging.Syslog import syslog
 
 
+
 def Update(l, stored_state):
     "Dispose of old vars and user options, mapping to new ones when suitable."
-    # No worry about entirely new vars because InitVars() takes care of them.
+    ZapOldVars(l)
+    NewVars(l)
     UpdateOldVars(l, stored_state)
     UpdateOldUsers(l)
     CanonicalizeUserOptions(l)
+    NewRequestsDatabase(l)
 
+
+
+def ZapOldVars(l):
+    if hasattr(l, 'num_spawns'):
+        del l.num_spawns
+
+
+
 uniqueval = []
 def UpdateOldVars(l, stored_state):
     """Transform old variable values into new ones, deleting old ones.
@@ -106,9 +119,11 @@ def UpdateOldVars(l, stored_state):
             if l.posters:
                 l.member_posting_only = 0
         delattr(l, "posters_includes_members")
-    else: # make sure everyone gets the behavior the list used to have
-        if l.posters:
-            l.member_posting_only = 0
+    elif l.data_version <= 10 and l.posters:
+        # make sure everyone gets the behavior the list used to have, but only
+        # for really old versions of Mailman (1.0b5 or before).  Any newer
+        # version of Mailman should not get this attribute whacked.
+        l.member_posting_only = 0
     #
     # transfer the list data type for holding members and digest members
     # to the dict data type starting file format version 11
@@ -156,6 +171,27 @@ def UpdateOldVars(l, stored_state):
             l.digest_members[k] = 0
 
 
+
+def NewVars(l):
+    """Add defaults for these new variables if they don't exist."""
+    def add_only_if_missing(attr, initval, l=l):
+        if not hasattr(l, attr):
+            setattr(l, attr, initval)
+    # 1.2 beta 1, baw 18-Feb-2000
+    # Autoresponder mixin class attributes
+    add_only_if_missing('autorespond_postings', 0, l)
+    add_only_if_missing('autorespond_admin', 0, l)
+    add_only_if_missing('autorespond_requests', 0, l)
+    add_only_if_missing('autoresponse_postings_text', '', l)
+    add_only_if_missing('autoresponse_admin_text', '', l)
+    add_only_if_missing('autoresponse_request_text', '', l)
+    add_only_if_missing('autoresponse_graceperiod', 90, l)
+    add_only_if_missing('postings_responses', {}, l)
+    add_only_if_missing('admin_responses', {}, l)
+    add_only_if_missing('reply_goes_to_list', '', l)
+
+
+
 def UpdateOldUsers(l):
     """Transform sense of changed user options."""
     # pre-1.0b11 to 1.0b11.  Force all keys in l.passwords to be lowercase
@@ -165,12 +201,15 @@ def UpdateOldUsers(l):
     l.passwords = passwords
 
 
+
 def CanonicalizeUserOptions(l):
     """Keys in user_options must be lower case."""
     # pre 1.0rc2 to 1.0rc3.  For all keys in l.user_options to be lowercase,
     # but merge options for both cases
     options = {}
     for k, v in l.user_options.items():
+        if k is None:
+            continue
         lcuser = string.lower(k)
         flags = 0
         if options.has_key(lcuser):
@@ -179,13 +218,48 @@ def CanonicalizeUserOptions(l):
         options[lcuser] = flags
     l.user_options = options
 
-def older(version, reference):
-    """True if version is older than current.
 
-    Different numbering systems imply version is older."""
-    if type(version) != type(reference):
-      return 1
-    if version >= reference:
-      return 0  
-    else:
-      return 1
+
+def NewRequestsDatabase(l):
+    """With version 1.2, we use a new pending request database schema."""
+    r = getattr(l, 'requests', {})
+    if not r:
+        # no old-style requests
+        return
+    for k, v in r.items():
+        if k == 'post':
+            # This is a list of tuples with the following format
+            #
+            # a sequential request id integer
+            # a timestamp float
+            # a message tuple: (author-email-str, message-text-str)
+            # a reason string
+            # the subject string
+            #
+            # We'll re-submit this as a new HoldMessage request, but we'll
+            # blow away the original timestamp and request id.  This means the
+            # request will live a little longer than it possibly should have,
+            # but that's no big deal.
+            for p in v:
+                author, text = p[2]
+                reason = p[3]
+                msg = Message.OutgoingMessage(text)
+                l.HoldMessage(msg, reason)
+            del r[k]
+        elif k == 'add_member':
+            # This is a list of tuples with the following format
+            #
+            # a sequential request id integer
+            # a timestamp float
+            # a digest flag (0 == nodigest, 1 == digest)
+            # author-email-str
+            # password
+            #
+            # See the note above; the same holds true.
+            for ign, ign, digest, addr, password in v:
+                l.HoldSubscription(addr, password, digest)
+            del r[k]
+        else:
+            syslog('error',
+                   "VERY BAD NEWS.  Unknown pending request type `%s' found"
+                   ' for list: %s' % (k, l.internal_name()))
